/***************************************************************************
 *   Copyright (C) 2004 by Stephen McInerney                               *
 *   spm@stedee.id.au                                                      *
 *                                                                         *
 *   $Id: visitors.h 41 2005-07-08 12:59:15Z steve $
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/***************************************************************************
 ***************************************************************************
 * visitors.h
 *
 ***************************************************************************
 ***************************************************************************/

/***************************************************************************
 ***************************************************************************
 * Modification History:
 ***********************
 * 19-Jun-2004 steve    Initial Creation
 * 03-Nov-2004 steve    Add "ico's" to the list of ignored file types
 * 04-Nov-2004 steve    Globalise re_filter'ing - for setting as option
 * 15-Apr-2005 steve    Increase Buffer size to 16K
 * 07-Jul-2005 steve    Fix use/setup of g_db_dirfilename & g_re_filter
 *                      Missing definitions in error_substring_extract
 * 08-Jul-2005 steve    Start adding definitions for non single char cmd
 *                        line options
 *
 ***************************************************************************
 ***************************************************************************/

#ifndef VISITORS_VISITORS_H
#define VISITORS_VISITORS_H 1

#include "common.h"
#include "db_visitors.h"
#include "error.h"
#include "messages.h"
#include "queries.h"
#include "xmalloc.h"

#define VISIT_TIME_SEP 1800                     /* Time between visits - 30 minutes */

#define OVECCOUNT 300
#define BUFSIZE  (16 * 2 * 512)                 /* Size of the line buffer - 16K */
#define BUFCUTOFF  125                          /* When printing out long buffers, cut off here */
#define MAX_RE_LENGTH 1024                      /* The max size in chars of an RE */
#define MAX_FILENAME_LENGTH 512                 /* The max size in chars of any specified filenames */

#define DAY_IN_SECONDS (60 * 60 * 24)

#define PATTERN_IGNORE_LIST "\\.(?:gif|css|jpe?g|png|js|ico)$"  /* Default list of file types to ignore */

#define PATTERN_NCSA "^([^ ]+) ([^ ]+) ([^ ]+) \\[([^ ]+) ([^ ]+)\\] \"([^ ]+) ([^ ]+) ?([^\"]*)\" ([^ ]+) ([^ ]+) \"([^\"]*)\" \"([^\"]+)\""
        /* Std Combined Log Format */
#define PATTERN_NCSA_COOKIE "^([^ ]+) ([^ ]+) ([^ ]+) \\[([^ ]+) ([^ ]+)\\] \"([^ ]+) ([^ ]+) ?([^\"]*)\" ([^ ]+) ([^ ]+) \"([^\"]*)\" \"([^\"]+)\" \"([^\"]+)\""
        /* CLF, + mod_usertrack cookie enclosed in quotes */

#define DATE_TIME_FORMAT "%d/%b/%Y:%H:%M:%S"    /* Default DATE_TIME format for input to strptime */

#define SIZE_ADDRESS 1024
#define SIZE_DATE_TIME 21
#define SIZE_REFERER 300
#define SIZE_BROWSER 512
#define SIZE_COOKIE 35

#define LF_NCSA_ADDRESS 1
#define LF_NCSA_USER 2
#define LF_NCSA_AUTHUSER 3
#define LF_NCSA_DATE_TIME 4
#define LF_NCSA_TIMEZONE 5
#define LF_NCSA_ACC_TYPE 6
#define LF_NCSA_URL 7
#define LF_NCSA_HTTP_VER 8
#define LF_NCSA_STATUS 9
#define LF_NCSA_BYTES 10
#define LF_NCSA_REFERER 11
#define LF_NCSA_BROWSER 12
#define LF_CUSTM_COOKIE 13

#define DB_CACHE_SIZE 20

/* Add command line options here - only those without single char abbreviations */
#define CMD_CLEANUP "cleanup"
#define CMD_QUERY "query"
#define CMD_Q_UNSEEN_SINCE "unseen-since"
#define CMD_Q_ACCESS_DELTA "access-delta"

/************************************************************************
 *                              STRUCTURES                              *
 ************************************************************************/
typedef struct {
    /* This is the raw data that will be stored for every cookie and address/browser combo */
    time_t first_access;                        /* When was the first access? */
    time_t last_access;                         /* When was the most recent access? */
    unsigned int nbr_pages;                     /* How many pages has this visitor seen - lifetime */
    unsigned int nbr_visits;                    /* How many visits has this visitor had - lifetime */
    time_t visit_start;                         /* When did this current visit start? */
    bool is_cookie;                             /* True: This record is a cookie, else is an address/browser combo */
    bool good_cookie;                           /* True: This cookie is a "good cookie". Has been seen before (eg filtered hit) */
} visit_data_type;

typedef struct {
    char cookie_key[SIZE_COOKIE];               /* The users cookie */
    visit_data_type data;                       /* visit data */
    char address_browser[SIZE_ADDRESS + SIZE_BROWSER];  /* The users address/browser combo, we will delete this alternate key if the cookie is Good */
} cookie_type;

typedef struct {
    char address_browser_key[SIZE_ADDRESS + SIZE_BROWSER];      /* The users address/browser combo */
    visit_data_type data;                       /* visit data */
} address_browser_type;

typedef struct cookie_list_element_ {
    char cookie_key[SIZE_COOKIE];               /* The users cookie */
    struct cookie_list_element_ *next;
} cookie_list_element;

/************************************************************************
 *                              GLOBALS                                 *
 ************************************************************************/
extern pcre *re_pattern_main;                   /* Main compiled RE */
extern pcre *re_pattern_filter;                 /* Filter RE, compiled, ignore these file types */
extern char g_re_filter[MAX_RE_LENGTH];         /* Filter RE, NOT compiled - RAW, ignore these file types */

/*-- DB Setup --*/
extern char g_db_dirfilename[MAX_FILENAME_LENGTH];      /* The name of the Database Directory Path/File to use for storage */

extern char *g_filename;                        /* Command Line specified file to open. */
extern time_t g_visittimeout;                   /* Visit Timeout in seconds */

/* Global Flags - what to do'isims */
extern bool flag_raw_stats_only;                /* Output a TAB separated single line of raw, non-derived results only */
extern int db_cache_size;                       /* Size of the DB Cache */
extern int g_cleanup_mode;                      /* Number of days beyond which older records will be deleted */


/************************************************************************
 *                              FUNCTIONS                               *
 ************************************************************************/
extern bool is_filtered (char *);               /* Is this a filtered URL? */

extern void re_compile_failed (int, const char *, char *);      /* Display a failed RE Compile & where */
extern void re_check_errors (int, unsigned long int, char *);   /* After an RE check, deal with any errors */
extern void error_substring_extract (char *, char *, int, int); /* Error when we fail on getting a substring */

extern void process_options (int, char *[]);    /* Process command line options */
extern time_t time_end_today (void);            /* Return the time in time_t format for the end of the current day */

#endif          /* VISITORS_VISITORS_H */


/************************************************************************
 ************************************************************************
 *                      END OF FILE                                     *
 ************************************************************************
 ************************************************************************/
